package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.vo.*;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * Created by lidr on 2021/4/14
 */
@Api(value = "数据资源分级分类表码管理")
@FeignClient(name = "dcuc-app", path = "/dcuc/app/dataResourceFacade")
public interface IDataResourceFacade {

    @ApiOperation(value = "数据分级表码列表")
    @GetMapping(value = "datalevels")
    List<DataLevelVo> getDataLevelList();

    @ApiOperation(value = "安全等级表码列表")
    @GetMapping(value = "securitylevels")
    List<DataLevelVo> getSecurityLevelList();

    @ApiOperation(value = "数据资源分类表码列表")
    @GetMapping(value = "dataclassifys")
    List<DataClassifyVo> getDataClassifyList();

    @ApiOperation(value = "字段分类表码列表")
    @GetMapping(value = "fieldclassifys")
    List<DataFieldClassifyVo> getFieldClassifyList();

    @ApiOperation(value = "数据资源树")
    @ApiImplicitParam(name = "attrType", value = "数据属性类型 TABLE：表 COLUMN：列")
    @GetMapping(value = "getAllDataResourceTree")
    List<DataResourceClassifyVo> getAllDataResourceTree(@RequestParam(value = "attrType", required = false) String attrType);

    @ApiOperation(value = "数据资源树")
    @ApiImplicitParam(name = "typeCode", value = "typeCode类型（数据分级：DATA_CLASSIFY；数据资源分类：DATA_RESOURCE_CLASSIFY；数据安全级别：DATA_SECURITY_LEVEL；字段分类：COLUMN_CLASSIFY）")
    @GetMapping(value = "getDataResourceTree")
    DataResourceClassifyVo getDataResourceTree(@RequestParam("typeCode") String typeCode);


    /**
     * 数据分级同步
     *
     * @return
     */
    @GetMapping(value = "dataLevelSync")
    ResponseStatus dataLevelSync();

    /**
     * 数据安全等级同步
     *
     * @return
     */
    @GetMapping(value = "dataSecSync")
    ResponseStatus dataSecSync();

    /**
     * 字段关系同步
     *
     * @return 状态
     */
    @GetMapping(value = "columnRelationSync")
    ResponseStatus columnRelationSync();

    /**
     * 数据资源分类同步
     *
     * @return
     */
    @GetMapping(value = "dataClaSync")
    ResponseStatus dataClaSync();

    /**
     * 字段分类同步
     *
     * @return
     */
    @GetMapping(value = "fieldClaSync")
    ResponseStatus fieldClaSync();

    /**
     * 表和字段同步
     *
     * @return 状态
     */
    @GetMapping(value = "syncTableAndColumn")
    ResponseStatus syncTableAndColumn();

    /**
     * 批量获取
     * 根据id和classifyType获取名字和code值
     *
     * @param ids
     * @param classifyType
     * @return
     */
    @PostMapping(value = "getDetailByIdsAndClaType")
    List<DataLevelVo> getDetailByIdsAndClaType(@RequestParam("ids") List<String> ids, @RequestParam("classifyType") String classifyType);

    /**
     * 根据id和classifyType获取名字和code值
     *
     * @param id
     * @param classifyType
     * @return
     */
    @PostMapping(value = "getDetailByIdAndClaType")
    DataLevelVo getDetailByIdAndClaType(@RequestParam("id") String id, @RequestParam("classifyType") String classifyType);
}
