package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.vo.ServiceResourceVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author huangzqa
 * @date 2021/3/1
 **/
@FeignClient(name = "dcuc-app", path = "/dcuc/app/serviceResourceFacade")
public interface IServiceResourceFacade {

    /**
     * 根据编码获取详情
     *
     * @param serviceCode 编码
     * @return 详情
     */
    @RequestMapping(value = "detailByCode")
    ServiceResource detailByCode(@RequestParam("serviceCode") String serviceCode);

    @PostMapping(value = "resourcePages")
    Page<ServiceResource> resourcePages(@RequestBody SearchDTO searchDTO);

    /**
     * 获取serviceCodes对应的服务资源
     *
     * @param list
     * @return
     */
    @PostMapping(value = "getByCodes")
    List<ServiceResourceDTO> getServiceByServiceCodes(@RequestBody List<String> list);

    /**
     * 服务资源列表分页查询
     * @return
     */
    @PostMapping(value = "/search")
    Page<ServiceResourceVo> page(@RequestBody SearchDTO toSearchDTO);

    /**
     * 服务资源导入
     * @param fileId
     * @return
     */
    @PostMapping(value = "/import")
    ResponseStatus impExcel(@RequestParam("fileId") String fileId);

    /**
     * 服务资源新增
     * @param serviceResourceDTO
     * @return
     */
    @PostMapping(value = "/service-resource")
    ResponseStatus save(@RequestBody ServiceResourceDTO serviceResourceDTO);

    /**
     * 服务资源详情
     * @param id
     * @return
     */
    @GetMapping(value = "/service-resource/{id}")
    ServiceResourceVo detailById(@PathVariable(name = "id") String id);
    /**
     * 服务资源修改
     * @param serviceResourceVo
     * @return
     */
    @PutMapping(value = "/service-resource")
    ResponseStatus updateServiceResource(@RequestBody ServiceResourceVo serviceResourceVo);

    /**
     * 服务资源删除
     * @param id
     * @return
     */
    @DeleteMapping(value = "/service-resource/{id}")
    ResponseStatus deleteById(@PathVariable(name = "id") String id);

    /**
     * 根据id集合获取
     * @param serviceIds
     * @return
     */
    @PostMapping("/service-resource/getByIds")
    List<ServiceResourceVo> getServiceByIds(@RequestBody List<String> serviceIds);
}
