package com.dragoninfo.dcuc.app.strategy;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.cons.ServiceStatusCons;
import com.dragoninfo.dcuc.app.dto.ResourceDTO;
import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.enumresources.ResourceTypeEnum;
import com.dragoninfo.dcuc.app.mapper.ServiceResourceMapper;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.TimeZone;

/**
 * @author mazq
 * @Description
 * @create 2020-11-25 14:21
 */
@Component
public class ServiceResourceStrategy implements ResourceOperateStrategy<ServiceResourceDTO ,ServiceResource> {

    @Autowired
    private ServiceResourceMapper serviceResourceMapper;

    @Override
    public ServiceResource insertRemoteSource(ServiceResourceDTO serviceResourceDTO) {
        LambdaQueryWrapper<ServiceResource> wrapper = Wrappers.lambdaQuery();
        wrapper.select()
               .eq(ServiceResource::getServiceCode,serviceResourceDTO.getServiceCode())
               .eq(ServiceResource::getDeleted,BooleanEnum.FALSE.getValue())
               .eq(ServiceResource::getResourceId, serviceResourceDTO.getResourceId());
        ServiceResource exist = serviceResourceMapper.selectOne(wrapper);
        ServiceResource serviceResource;
        if (null == exist) {
            serviceResource = new ServiceResource();
            //如果是删除状态的服务资源不插入数据库
            if(ServiceStatusCons.STATUS_DELETED.equals(serviceResourceDTO.getServiceStatus())) {
                return serviceResource;
            }
            BeanUtil.copyProperties(serviceResourceDTO, serviceResource);
            serviceResource.setCreateTime(new Date());
            serviceResourceMapper.insert(serviceResource);
        } else {
            serviceResource = exist;
            BeanUtil.copyProperties(serviceResourceDTO, serviceResource, "id","createTime");
            serviceResource.setUpdateTime(new Date());
            serviceResourceMapper.updateById(serviceResource);
            //如果是删除状态的服务资源,删除
            if (ServiceStatusCons.STATUS_DELETED.equals(serviceResourceDTO.getServiceStatus())) {
                serviceResourceMapper.deleteById(serviceResource.getId());
            }
        }
        return serviceResource;
    }

    @Override
    public boolean canHandle(String resourceId) {
       return ResourceTypeEnum.TJ_SERVICE.getResourceId().equals(resourceId) ||
                ResourceTypeEnum.BU_SERVICE.getResourceId().equals(resourceId);
    }

}
