CREATE TABLE T_APP_DATA_CLA(
    ID VARCHAR2(32) NOT NULL,
    FIR_ID VARCHAR2(32),
    SEC_ID VARCHAR2(32),
    LAB_CLA_ID VARCHAR2(32),
    CODE NVARCHAR2(32),
    NAME NVARCHAR2(100),
    DATA_TYPE NVARCHAR2(100),
    LEVEL NVARCHAR2(10),
    TYPE NVARCHAR2(10),
    REMARK NVARCHAR2(128),
    DELETED NVARCHAR2(5),
    CREATE_TIME DATE,
    CREATE_USER VARCHAR2(32),
    UPDATE_TIME DATE,
    UPDATE_USER VARCHAR2(32),
    DELETE_TIME DATE,
    DELETE_USER VARCHAR2(32),
    PRIMARY KEY (ID)
);

COMMENT ON TABLE T_APP_DATA_CLA IS '数据资源分类表';
COMMENT ON COLUMN T_APP_DATA_CLA.ID IS '主键';
COMMENT ON COLUMN T_APP_DATA_CLA.FIR_ID IS '一级id';
COMMENT ON COLUMN T_APP_DATA_CLA.SEC_ID IS '二级id';
COMMENT ON COLUMN T_APP_DATA_CLA.LAB_CLA_ID IS '标签分类id';
COMMENT ON COLUMN T_APP_DATA_CLA.CODE IS '码值';
COMMENT ON COLUMN T_APP_DATA_CLA.NAME IS '名称';
COMMENT ON COLUMN T_APP_DATA_CLA.DATA_TYPE IS '数据授权资源类型（第三方提供）';
COMMENT ON COLUMN T_APP_DATA_CLA.LEVEL IS '层级';
COMMENT ON COLUMN T_APP_DATA_CLA.TYPE IS '分类类型 一级分类 二级分类 标签分类 标签';
COMMENT ON COLUMN T_APP_DATA_CLA.REMARK IS '说明';
COMMENT ON COLUMN T_APP_DATA_CLA.DELETED IS '是否删除';
COMMENT ON COLUMN T_APP_DATA_CLA.CREATE_TIME IS '新增时间';
COMMENT ON COLUMN T_APP_DATA_CLA.CREATE_USER IS '新增人员账号';
COMMENT ON COLUMN T_APP_DATA_CLA.UPDATE_TIME IS '修改时间';
COMMENT ON COLUMN T_APP_DATA_CLA.UPDATE_USER IS '更新人账号';
COMMENT ON COLUMN T_APP_DATA_CLA.DELETE_TIME IS '删除时间';
COMMENT ON COLUMN T_APP_DATA_CLA.DELETE_USER IS '删除人账号';

CREATE INDEX IDX_T_APP_DATA_CLA_FIR ON T_APP_DATA_CLA(FIR_ID);

CREATE INDEX IDX_T_APP_DATA_CLA_SEC ON T_APP_DATA_CLA(SEC_ID);

CREATE INDEX IDX_T_APP_DATA_CLA_LAB_CLA_ID ON T_APP_DATA_CLA(LAB_CLA_ID);

CREATE INDEX IDX_T_APP_DATA_CODE ON T_APP_DATA_CLA(CODE);

CREATE TABLE T_APP_DATA_LEVEL(
    ID VARCHAR2(32) NOT NULL,
    LEVEL_CODE NVARCHAR2(32),
    LEVEL_NAME NVARCHAR2(100),
    DATA_TYPE NVARCHAR2(100),
    REMARK NVARCHAR2(128),
    DELETED NVARCHAR2(32),
    CREATE_TIME DATE,
    CREATE_USER VARCHAR2(32),
    UPDATE_TIME DATE,
    UPDATE_USER VARCHAR2(32),
    DELETE_TIME DATE,
    DELETE_USER VARCHAR2(32),
    PRIMARY KEY (ID)
);

COMMENT ON TABLE T_APP_DATA_LEVEL IS '数据分级表';
COMMENT ON COLUMN T_APP_DATA_LEVEL.ID IS '主键';
COMMENT ON COLUMN T_APP_DATA_LEVEL.LEVEL_CODE IS '码值';
COMMENT ON COLUMN T_APP_DATA_LEVEL.LEVEL_NAME IS '名称';
COMMENT ON COLUMN T_APP_DATA_LEVEL.DATA_TYPE IS '数据授权资源类型（第三方提供）';
COMMENT ON COLUMN T_APP_DATA_LEVEL.REMARK IS '说明';
COMMENT ON COLUMN T_APP_DATA_LEVEL.DELETED IS '是否删除';
COMMENT ON COLUMN T_APP_DATA_LEVEL.CREATE_TIME IS '新增时间';
COMMENT ON COLUMN T_APP_DATA_LEVEL.CREATE_USER IS '新增人员账号';
COMMENT ON COLUMN T_APP_DATA_LEVEL.UPDATE_TIME IS '修改时间';
COMMENT ON COLUMN T_APP_DATA_LEVEL.UPDATE_USER IS '更新人账号';
COMMENT ON COLUMN T_APP_DATA_LEVEL.DELETE_TIME IS '删除时间';
COMMENT ON COLUMN T_APP_DATA_LEVEL.DELETE_USER IS '删除人账号';

CREATE INDEX IDX_T_APP_DATA_LEVEL_LEVEL_CODE ON T_APP_DATA_LEVEL(LEVEL_CODE);



CREATE TABLE T_APP_DATA_SEC(
    ID VARCHAR2(32) NOT NULL,
    SEC_CODE NVARCHAR2(32),
    SEC_NAME NVARCHAR2(100),
    DATA_TYPE NVARCHAR2(100),
    REMARK NVARCHAR2(128),
    DELETED NVARCHAR2(32),
    CREATE_TIME DATE,
    CREATE_USER VARCHAR2(32),
    UPDATE_TIME DATE,
    UPDATE_USER VARCHAR2(32),
    DELETE_TIME DATE,
    DELETE_USER VARCHAR2(32),
    PRIMARY KEY (ID)
);

COMMENT ON TABLE T_APP_DATA_SEC IS '数据安全等级表';
COMMENT ON COLUMN T_APP_DATA_SEC.ID IS '主键';
COMMENT ON COLUMN T_APP_DATA_SEC.SEC_CODE IS '码值';
COMMENT ON COLUMN T_APP_DATA_SEC.SEC_NAME IS '名称';
COMMENT ON COLUMN T_APP_DATA_SEC.DATA_TYPE IS '数据授权资源类型（第三方提供）';
COMMENT ON COLUMN T_APP_DATA_SEC.REMARK IS '说明';
COMMENT ON COLUMN T_APP_DATA_SEC.DELETED IS '是否删除';
COMMENT ON COLUMN T_APP_DATA_SEC.CREATE_TIME IS '新增时间';
COMMENT ON COLUMN T_APP_DATA_SEC.CREATE_USER IS '新增人员账号';
COMMENT ON COLUMN T_APP_DATA_SEC.UPDATE_TIME IS '修改时间';
COMMENT ON COLUMN T_APP_DATA_SEC.UPDATE_USER IS '更新人账号';
COMMENT ON COLUMN T_APP_DATA_SEC.DELETE_TIME IS '删除时间';
COMMENT ON COLUMN T_APP_DATA_SEC.DELETE_USER IS '删除人账号';

CREATE INDEX IDX_T_APP_DATA_LEVEL_SEC_CODE ON T_APP_DATA_SEC(SEC_CODE);

CREATE TABLE T_APP_FIELD_CLA(
    ID VARCHAR2(32) NOT NULL,
    FIR_ID VARCHAR2(32),
    CODE NVARCHAR2(32),
    NAME NVARCHAR2(100),
    DATA_TYPE NVARCHAR2(100),
    LEVEL NVARCHAR2(32),
    TYPE NVARCHAR2(32),
    REMARK NVARCHAR2(128),
    DELETED NVARCHAR2(32),
    CREATE_TIME DATE,
    CREATE_USER VARCHAR2(32),
    UPDATE_TIME DATE,
    UPDATE_USER VARCHAR2(32),
    DELETE_TIME DATE,
    DELETE_USER VARCHAR2(32),
    PRIMARY KEY (ID)
);

COMMENT ON TABLE T_APP_FIELD_CLA IS '字段分类分类表';
COMMENT ON COLUMN T_APP_FIELD_CLA.ID IS '主键';
COMMENT ON COLUMN T_APP_FIELD_CLA.FIR_ID IS '一级id';
COMMENT ON COLUMN T_APP_FIELD_CLA.CODE IS '码值';
COMMENT ON COLUMN T_APP_FIELD_CLA.NAME IS '名称';
COMMENT ON COLUMN T_APP_FIELD_CLA.DATA_TYPE IS '数据授权资源类型（第三方提供）';
COMMENT ON COLUMN T_APP_FIELD_CLA.LEVEL IS '层级';
COMMENT ON COLUMN T_APP_FIELD_CLA.TYPE IS '分类类型 一级分类 二级分类';
COMMENT ON COLUMN T_APP_FIELD_CLA.REMARK IS '说明';
COMMENT ON COLUMN T_APP_FIELD_CLA.DELETED IS '是否删除';
COMMENT ON COLUMN T_APP_FIELD_CLA.CREATE_TIME IS '新增时间';
COMMENT ON COLUMN T_APP_FIELD_CLA.CREATE_USER IS '新增人员账号';
COMMENT ON COLUMN T_APP_FIELD_CLA.UPDATE_TIME IS '修改时间';
COMMENT ON COLUMN T_APP_FIELD_CLA.UPDATE_USER IS '更新人账号';
COMMENT ON COLUMN T_APP_FIELD_CLA.DELETE_TIME IS '删除时间';
COMMENT ON COLUMN T_APP_FIELD_CLA.DELETE_USER IS '删除人账号';

CREATE INDEX IDX_T_APP_FIELD_CLA_FIR ON T_APP_FIELD_CLA(FIR_ID);

CREATE INDEX IDX_T_APP_FIELD_CLA_CODE ON T_APP_FIELD_CLA(CODE);
ALTER TABLE "T_APPLY_INFO" ADD "org_code"  VARCHAR2(100);
ALTER TABLE "T_APPLY_INFO" MODIFY "MANAGER_DESCRIBE" VARCHAR2(800);